@@ -13,12 +13,12 @@
 import contextlib
 import time
 
-from keystoneclient import adapter
-from keystoneclient.auth.identity import v2 as v2_auth
-from keystoneclient.auth.identity import v3 as v3_auth
-from keystoneclient import discover
-from keystoneclient import exceptions as ks_exc
-from keystoneclient import session
+from keystoneauth1 import adapter
+from keystoneauth1 import discover
+from keystoneauth1 import exceptions as ka_exc
+from keystoneauth1.identity import v2 as v2_auth
+from keystoneauth1.identity import v3 as v3_auth
+from keystoneauth1 import session
 from oslo_utils import strutils
 import six.moves.urllib.parse as urlparse
 
@@ -35,10 +35,10 @@ def _discover_auth_versions(session, auth_url):
     v2_auth_url = None
     v3_auth_url = None
     try:
-        ks_discover = discover.Discover(session=session, auth_url=auth_url)
+        ks_discover = discover.Discover(session=session, url=auth_url)
         v2_auth_url = ks_discover.url_for('2.0')
         v3_auth_url = ks_discover.url_for('3.0')
-    except ks_exc.DiscoveryFailure:
+    except ka_exc.DiscoveryFailure:
         raise
     except exceptions.ClientException:
         # Identity service may not support discovery. In that case,
@@ -71,6 +71,7 @@ def _get_keystone_session(**kwargs):
     auth_url = kwargs.pop('auth_url', None)
     project_id = kwargs.pop('project_id', None)
     project_name = kwargs.pop('project_name', None)
+    token = kwargs['token']
     timeout = kwargs.get('timeout')
 
     if insecure:
@@ -100,7 +101,21 @@ def _get_keystone_session(**kwargs):
     use_v3 = v3_auth_url and (use_domain or (not v2_auth_url))
     use_v2 = v2_auth_url and not use_domain
 
-    if use_v3:
+    if use_v3 and token:
+        auth = v3_auth.Token(
+            v3_auth_url,
+            token=token,
+            project_name=project_name,
+            project_id=project_id,
+            project_domain_name=project_domain_name,
+            project_domain_id=project_domain_id)
+    elif use_v2 and token:
+        auth = v2_auth.Token(
+            v2_auth_url,
+            token=token,
+            tenant_id=project_id,
+            tenant_name=project_name)
+    elif use_v3:
         # the auth_url as v3 specified
         # e.g. http://no.where:5000/v3
         # Keystone will return only v3 as viable option
@@ -125,6 +140,7 @@ def _get_keystone_session(**kwargs):
             kwargs.pop('password', None),
             tenant_id=project_id,
             tenant_name=project_name)
+
     else:
         raise exc.CommandError('Unable to determine the Keystone version '
                                'to authenticate with using the given '
@@ -160,16 +176,20 @@ class AuthPlugin(auth.BaseAuthPlugin):
     def __init__(self, auth_system=None, **kwargs):
         self.opt_names.extend(self.common_opt_names)
         super(AuthPlugin, self).__init__(auth_system, **kwargs)
+        # NOTE(sileht): backward compat
+        if self.opts.get('auth_token') and not self.opts.get('token'):
+            self.opts['token'] = self.opts.get('auth_token')
 
     def _do_authenticate(self, http_client):
-        token = self.opts.get('token') or self.opts.get('auth_token')
+        token = self.opts.get('token')
         endpoint = self.opts.get('endpoint')
-        if not (token and endpoint):
+        if not (endpoint and token):
             ks_kwargs = self._get_ks_kwargs(http_timeout=http_client.timeout)
             ks_session = _get_keystone_session(**ks_kwargs)
-            token = lambda: ks_session.get_token()
-            endpoint = (self.opts.get('endpoint') or
-                        _get_endpoint(ks_session, **ks_kwargs))
+            if not token:
+                token = lambda: ks_session.get_token()
+            if not endpoint:
+                endpoint = _get_endpoint(ks_session, **ks_kwargs)
         self.opts['token'] = token
         self.opts['endpoint'] = endpoint
 
@@ -178,6 +198,7 @@ class AuthPlugin(auth.BaseAuthPlugin):
                       self.opts.get('tenant_id'))
         project_name = (self.opts.get('project_name') or
                         self.opts.get('tenant_name'))
+        token = self.opts.get('token')
         ks_kwargs = {
             'username': self.opts.get('username'),
             'password': self.opts.get('password'),
@@ -198,6 +219,7 @@ class AuthPlugin(auth.BaseAuthPlugin):
             'service_type': self.opts.get('service_type'),
             'region_name': self.opts.get('region_name'),
             'timeout': http_timeout,
+            'token': token() if callable(token) else token,
         }
         return ks_kwargs
 
@@ -212,8 +234,7 @@ class AuthPlugin(auth.BaseAuthPlugin):
 
         :raises: AuthPluginOptionsMissing
         """
-        has_token = self.opts.get('token') or self.opts.get('auth_token')
-        no_auth = has_token and self.opts.get('endpoint')
+        has_token = self.opts.get('token')
         has_project_domain_or_tenant = (self.opts.get('project_id') or
                                         (self.opts.get('project_name') and
                                         (self.opts.get('user_domain_name') or
@@ -224,7 +245,7 @@ class AuthPlugin(auth.BaseAuthPlugin):
                           and has_project_domain_or_tenant
                           and self.opts.get('password')
                           and self.opts.get('auth_url'))
-        missing = not (no_auth or has_credential)
+        missing = not (has_token or has_credential)
         if missing:
             missing_opts = []
             opts = ['token', 'endpoint', 'username', 'password', 'auth_url',
@@ -267,7 +288,7 @@ def _adjust_kwargs(kwargs):
         if timeout <= 0:
             timeout = None
 
-    insecure = strutils.bool_from_string(kwargs.get('insecure'))
+    insecure = strutils.bool_from_string(client_kwargs.get('insecure'))
     verify = kwargs.get('verify')
     if verify is None:
         if insecure:
@@ -367,7 +388,7 @@ def get_auth_plugin(endpoint, **kwargs):
     return auth_plugin
 
 
-LEGACY_OPTS = ('auth_plugin', 'auth_url', 'token', 'insecure',  'cacert',
+LEGACY_OPTS = ('auth_plugin', 'auth_url', 'token', 'insecure', 'cacert',
                'tenant_id', 'project_id', 'username', 'password',
                'project_name', 'tenant_name',
                'user_domain_name', 'user_domain_id',
@@ -381,6 +402,8 @@ def _construct_http_client(**kwargs):
         # Drop legacy options
         for opt in LEGACY_OPTS:
             kwargs.pop(opt, None)
+        # Drop aodh_endpoint from kwargs
+        kwargs.pop('aodh_endpoint', None)
 
         return SessionClient(
             session=kwargs.pop('session'),
@@ -389,7 +412,7 @@ def _construct_http_client(**kwargs):
                                                          'publicURL')),
             region_name=kwargs.pop('region_name', None),
             user_agent=kwargs.pop('user_agent', 'python-ceilometerclient'),
-            auth=kwargs.get('auth', None),
+            auth=kwargs.get('auth'),
             timings=kwargs.pop('timings', None),
             **kwargs)
     else:
